﻿using igGrid_EditingEvents.Models;
using Infragistics.Web.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using igGrId_EditingEvents.Models;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace igGrid_EditingEvents.Controllers
{
    public class HomeController : Controller
    {
        ProductList pl = new ProductList();
        ProductGroupList pgl = new ProductGroupList();

        // GET: Home
        public ActionResult Index()
        {
            GridModel model = CreateProductsGrid();
            return View(model);
        }

        public JsonResult GetData_Generic(string path, string layout)
        {
            GridModel model = CreateProductsGrid();

            model.DataSource = pl.GetProductList();

            if (layout == null)
            {
                return model.GetData();
            }

            return model.GetData(path, layout);
        }

        //[ComboDataSourceAction]
        public ActionResult GetDistict_Generic(string path, string filter)
        {
            IList<ComboDataViewModel> results = new List<ComboDataViewModel>();

            foreach (var group in pgl.GetProductGroupList())
            {
                ComboDataViewModel dataViewModel = new ComboDataViewModel(group.ProductGroupId, group.ProductGroupCode);
                results.Add(dataViewModel);
            }

            // ReSharper disable once Mvc.ViewNotResolved
            return View(results.AsQueryable());
        }

        [HttpPost]
        public ActionResult SaveChanges_Generic(string path, string a_TableProperty)
        {
            List<Transaction<JObject>> transactions = GetGridTransactions(this);

            foreach (Transaction<JObject> t in transactions)
            {
                foreach (var column in t.row)
                {
                    if (column.Key == "StartDate")
                    {
                        Console.WriteLine(column.Value);
                    }
                }
            }

            return new JsonResult
            {
                Data = new
                {
                    Success = true,
                    Type = "Successful"

                },
                JsonRequestBehavior = JsonRequestBehavior.AllowGet
            };
        }

        private GridModel CreateProductsGrid()
        {
            GridModel gridModel = new GridModel();

            gridModel.DataSourceUrl = Url.Action("GetData_Generic", "Home");
            gridModel.ID = "grid1";
            gridModel.AutoGenerateLayouts = false;
            gridModel.AutoGenerateColumns = false;
            gridModel.AutoCommit = true;
            gridModel.Width = "800px";
            gridModel.Height = "400px";
            gridModel.PrimaryKey = "Id";
            
            GridColumn col1 = new GridColumn("Id", "Id", "number", "auto");    
            gridModel.Columns.Add(col1);

            GridColumn col0 = new GridColumn("Field0", "Field0", "string", "auto");
            gridModel.Columns.Add(col0);

            GridColumn col2 = new GridColumn("Field1", "Field1", "numeric", "auto");
            col2.Format = "0.0000000000";
            col2.Template = "<div style='text-align: right;'>${" + col2.Key + "}</div>";
            gridModel.Columns.Add(col2);

            GridColumn col3 = new GridColumn("Field2", "Field2", "numeric", "auto");
            col3.Format = "0.0000000000";
            col3.Template = "<div style='text-align: right;'>${" + col3.Key + "}</div>";
            gridModel.Columns.Add(col3);

            GridColumn col4 = new GridColumn("Field1ReadOnly", "Field1ReadOnly", "bool", "auto");
            col4.Hidden = true;
            gridModel.Columns.Add(col4);

            GridColumn col5 = new GridColumn("Field2ReadOnly", "Field2ReadOnly", "bool", "auto");
            col5.Hidden = true;
            gridModel.Columns.Add(col5);

            //GridColumn col4 = new GridColumn("Product Group Code", "ProductGroupId", "number", "auto");
            //col4.FormatterFunction = "lookupGridColumnName";
            //gridModel.Columns.Add(col4);

            //GridColumn col5 = new GridColumn("StartDate", "StartDate", "date", "auto");
            //col5.Format = "MM/dd/yyyy";

            //col5.Template = "{{if ${" + col5.Key + "} == '01/01/1900' }}" +
            //                    "<div>00/00/0000</div>" +
            //                    "{{else}}" +
            //                    "<div>${" + col5.Key + "}</div>" +
            //                    "{{/if}}";
            //gridModel.Columns.Add(col5);

            GridUpdating updating = new GridUpdating();
            updating.EditMode = GridEditMode.Row;
            //updating.RowEditDialogOptions.Height = "auto";
            //updating.RowEditDialogOptions.DialogTemplateSelector = "#dialogTemplate";
            //updating.RowEditDialogOptions.EditorsTemplateSelector = "#editorsTemplate";
            updating.EnableAddRow = true;
            gridModel.UpdateUrl = Url.Action("SaveChanges_Generic", "Home", new { a_TableProperty = "Products" });
            gridModel.Features.Add(updating);

            ColumnUpdatingSetting updatingSetting = new ColumnUpdatingSetting();
            updatingSetting.ColumnKey = "Id";
            updatingSetting.ReadOnly = true;
            updating.ColumnSettings.Add(updatingSetting);

            ColumnUpdatingSetting updatingSetting2 = new ColumnUpdatingSetting();
            updatingSetting2.ColumnKey = "Field0";
            updatingSetting2.ReadOnly = false;
            updating.ColumnSettings.Add(updatingSetting2);

            ColumnUpdatingSetting updatingSetting3 = new ColumnUpdatingSetting();
            updatingSetting3.ColumnKey = "Field1";
            updatingSetting3.ReadOnly = false;
            updatingSetting3.NumericEditorOptions.ReadOnly = false;
			//updatingSetting3.NumericEditorOptions.MinDecimals = 0;
            //updatingSetting3.Required = true;
            //updatingSetting3.NumericEditorOptions.ValidatorOptions.Required = new ValidatorRule("This field is required.");
            //updatingSetting3.NumericEditorOptions.ValidatorOptions.RequiredIndication = true;
            //updatingSetting3.NumericEditorOptions.ValidatorOptions.OnBlur = false;
            //updatingSetting3.NumericEditorOptions.ValidatorOptions.OnChange = false;
            //updatingSetting3.NumericEditorOptions.DataMode = NumericEditorDataMode.Double;
            //updatingSetting3.NumericEditorOptions.MinDecimals = 2;
            //updatingSetting3.NumericEditorOptions.MaxDecimals = 10;
            //updatingSetting3.NumericEditorOptions.MaxValue = 99.999999999;
            updating.ColumnSettings.Add(updatingSetting3);

            ColumnUpdatingSetting updatingSetting4 = new ColumnUpdatingSetting();
            updatingSetting4.ColumnKey = "Field2";
            updatingSetting4.ReadOnly = false;
            updatingSetting4.NumericEditorOptions.ReadOnly = false;
            updatingSetting4.Required = true;
            //updatingSetting4.NumericEditorOptions.ValidatorOptions.Required = new ValidatorRule("This field is required.");
            //updatingSetting4.NumericEditorOptions.ValidatorOptions.RequiredIndication = true;
            //updatingSetting4.NumericEditorOptions.ValidatorOptions.OnBlur = true;
            //updatingSetting4.NumericEditorOptions.ValidatorOptions.OnChange = false;
            //updatingSetting4.NumericEditorOptions.DataMode = NumericEditorDataMode.Double;
            //updatingSetting4.NumericEditorOptions.MinDecimals = 2;
            //updatingSetting4.NumericEditorOptions.MaxDecimals = 10;
            //updatingSetting4.NumericEditorOptions.MaxValue = 99.999999999;
            updating.ColumnSettings.Add(updatingSetting4);

            //ColumnUpdatingSetting updatingSetting = new ColumnUpdatingSetting();
            //updatingSetting.ColumnKey = "ProductGroupId";
            //updatingSetting.EditorType = ColumnEditorType.Combo;
            //updatingSetting.ComboEditorOptions.FilteringType = ComboFilteringType.Remote;
            //updatingSetting.ComboEditorOptions.FilterExprUrlKey = "filter";
            //updatingSetting.ComboEditorOptions.LoadOnDemandSettings = new ComboLoadOnDemandSettings() { Enabled = true, PageSize = 25 };
            //updatingSetting.ComboEditorOptions.TextKey = "Value";
            //updatingSetting.ComboEditorOptions.ValueKey = "Key";
            //updatingSetting.ComboEditorOptions.Virtualization = true;
            //updatingSetting.ComboEditorOptions.DataSourceUrl = Url.Action("GetDistict_Generic", "Home");
            //updating.ColumnSettings.Add(updatingSetting);

            //ColumnUpdatingSetting updatingSetting2 = new ColumnUpdatingSetting();
            //updatingSetting2.ColumnKey = "StartDate";
            //updatingSetting2.EditorType = ColumnEditorType.DatePicker;
            //updatingSetting2.DateEditorOptions.ValidatorOptions.OnSubmit = true;
            //updatingSetting2.DateEditorOptions.ValidatorOptions.OnBlur = true;
            //updatingSetting2.DateEditorOptions.DataMode = DateEditorDataMode.Date;
            ////updatingSetting2.DateEditorOptions.MinValue = Constants.MinDate.AddDays(-1);
            ////updatingSetting2.DateEditorOptions.MaxValue = Constants.MaxDate;
            //updatingSetting2.DateEditorOptions.DataMode = DateEditorDataMode.Date;
            //updatingSetting2.DateEditorOptions.ValidatorOptions.Required = new ValidatorRule("This field is required.");
            //updatingSetting2.DateEditorOptions.ValidatorOptions.RequiredIndication = true;
            //updatingSetting2.DateEditorOptions.ButtonType = TextEditorButtonType.DropDown;
            //updating.ColumnSettings.Add(updatingSetting2);

            //GridSelection selectionFeature = new GridSelection();
            //selectionFeature.MultipleSelection = false;
            //selectionFeature.Mode = SelectionMode.Row;
            //gridModel.Features.Add(selectionFeature);

            ////Filtering
            //GridFiltering filtering = new GridFiltering();
            //filtering.Mode = FilterMode.Advanced; //Must use Advanced filtering with Virtualization
            //filtering.AdvancedModeHeaderButtonLocation = "left";
            //filtering.FilterDialogWidth = "490px";
            //filtering.FilterDialogColumnDropDownDefaultWidth = "190px";
            //filtering.FilterDialogFilterDropDownDefaultWidth = "115px";
            //filtering.FilterDialogExprInputDefaultWidth = "115px";
            //filtering.CaseSensitive = true; //Must be set to true. Otherwise, LOWER is used on the database field which significantly slows the query down for large tables.
            //filtering.FilterSummaryAlwaysVisible = false; //Should be set to false.  There is a bug where there are two filter summaries if the filter column settings default expressions is set.
            //gridModel.Features.Add(filtering);

            ////Paging
            //GridPaging paging = new GridPaging();
            //paging.PageSize = 250;
            //paging.Type = OpType.Remote;
            //paging.ShowPageSizeDropDown = true;
            //paging.PageSizeList.AddRange(new int[] { 15, 50, 250, 500, 1000, 5000 });
            //gridModel.Features.Add(paging);


            ////Virtualization
            //gridModel.RowVirtualization = true;
            //gridModel.ColumnVirtualization = false;
            //gridModel.VirtualizationMode = VirtualizationMode.Continuous;  //VirtualizationMode.Fixed doesn't seem to work correctly

            //Date settings
            //gridModel.EnableUTCDates = true;
            //updatingSetting2.DateEditorOptions.EnableUTCDates = true;
            //col5.DateDisplayType = DateDisplayType.UTC;

            return gridModel;
        }

        private const string c_IgFormDataKey = "ig_transactions";

        public static List<Transaction<JObject>> GetGridTransactions(Controller a_Controller)
        {
            return GetGridTransactions<JObject>(a_Controller);
        }


        public static List<Transaction<T>> GetGridTransactions<T>(Controller a_Controller) where T : class
        {
            a_Controller.ViewData["GenerateCompactJSONResponse"] = true;

            return JsonConvert.DeserializeObject<List<Transaction<T>>>(a_Controller.HttpContext.Request.Form[c_IgFormDataKey]);
        }

        //[GridDataSourceAction]
        //public ActionResult GetGridData()
        //{
        //    //return View(igGrid_MVC.Models.ProductsList.GetGridData());
        //    return View(pl.GetProductList());
        //}
    }
}